@@ -1,7 +1,6 @@
 import contextlib
-from locale import D_FMT
 import sys
-import contextlib
+from locale import D_FMT
 
 if sys.version_info[:2] >= (3, 8):
     # TODO: Import directly (no need for conditional) when `python_requires = >= 3.8`
@@ -18,6 +17,63 @@ except PackageNotFoundError:  # pragma: no cover
 finally:
     del version, PackageNotFoundError
 
+
+_OPTIONAL_IMPORTS_FOR_OCE_ONLINE = [
+    "sklearn",
+    "torch",
+    "torch_geometric",
+    "torch_scatter",
+    "torch_sparse",
+    "torch_cluster",
+    "torch_spline_conv",
+    "rdkit",
+    "sklearn.metrics",
+    "scipy",
+    "scipy.stats",
+    "joblib",
+    "sklearn.model_selection",
+    "tqdm",
+    "sklearn.preprocessing",
+    "rdkit.Chem",
+    "torch.nn",
+    "torch_geometric.nn",
+    "torch_geometric.data",
+    "torch.optim",
+    "torch_geometric.utils",
+    "torch.nn.functional",
+    "torch_geometric.nn.inits",
+    "rdkit.DataStructs",
+    "rdkit.DataStructs.cDataStructs",
+    "rdkit.Chem.AtomPairs",
+    "rdkit.Chem.AtomPairs.Sheridan",
+    "rdkit.Chem.Pharm2D",
+    "torch_geometric.loader",
+    "torch_geometric.data.data",
+    "torch.utils",
+    "torch.utils.data",
+    "torch.autograd",
+    "rdkit.Chem.rdchem",
+    "pytorch_lightning",
+    "rdkit.Chem.Scaffolds",
+    "PIL",
+    "selfies",
+    "sklearn.cross_decomposition",
+    "sklearn.decomposition",
+    "sklearn.neighbors",
+    "sklearn.manifold",
+    "hyperopt",
+    "hyperopt.pyll",
+]
+
+for imp in _OPTIONAL_IMPORTS_FOR_OCE_ONLINE:
+    try:
+        __import__(imp)
+    except ImportError:
+        import sys
+        from unittest.mock import MagicMock
+
+        sys.modules[imp] = MagicMock()
+
 from rdkit import RDLogger
 
 RDLogger.DisableLog("rdApp.*")
@@ -38,22 +94,28 @@ if not path.exists(path.join(path.expanduser("~"), f".oce/cache/")):
 if not path.exists(path.join(path.expanduser("~"), f".oce/cache/vecrep/")):
     os.mkdir(path.join(path.expanduser("~"), f".oce/cache/vecrep/"))
 
-import pandas as pd
-
 import json
 
+import pandas as pd
 
 CONFIG_PATH = os.path.join(os.path.expanduser("~"), ".oce/CONFIG.json")
 if os.path.exists(CONFIG_PATH):
     with open(CONFIG_PATH) as f:
         CONFIG_ = json.load(f)
 else:
-    CONFIG_ = {"MAP_LOCATION": "cpu", "USE_CUDA": False, "CDD_TOKEN": None, "VAULT_ID": None, "NUM_WORKERS": 4}
+    CONFIG_ = {
+        "MAP_LOCATION": "cpu",
+        "USE_CUDA": False,
+        "CDD_TOKEN": None,
+        "VAULT_ID": None,
+        "NUM_WORKERS": 0,
+    }
     with open(CONFIG_PATH, "w+") as f:
         json.dump(CONFIG_, f)
 
 CONFIG = CONFIG_.copy()
 
+
 def update_config():
     """Update the configuration file.
 
@@ -65,6 +127,7 @@ def update_config():
 
     with contextlib.suppress(ImportError):
         import torch
+
         CONFIG["DEVICE"] = torch.device(CONFIG["MAP_LOCATION"])
         CONFIG["USE_CUDA"] = "cuda" in CONFIG["MAP_LOCATION"]
 
@@ -95,37 +158,51 @@ def remove_config_param(param):
 
 update_config()
 
+
 def ExampleDataFrame():
-    return pd.read_csv("gs://oloren-public-data/sample-csvs/sample_data3.csv")
+    return pd.read_csv(
+        "https://storage.googleapis.com/oloren-public-data/sample-csvs/sample_data3.csv"
+    )
+
 
 from .base_class import *
 from .basics import *
-from .ensemble import *
-from .gnn import *
-from .representations import *
-from .external import *
-from .splitters import *
+from .benchmarks import *
 from .dataset import *
+from .ensemble import *
+from .external import *
+from .gnn import *
+from .hyperparameters import *
+from .internal import *
 from .interpret import *
 from .manager import *
-from .visualizations import *
+from .representations import *
+from .splitters import *
 from .uncertainty import *
-from .benchmarks import *
-from .hyperparameters import *
+from .visualizations import *
+
 
 def ExampleDataset():
-    if os.path.exists(path.join(path.expanduser("~"), f".oce/exampledataset.oce")):
-        return load(path.join(path.expanduser("~"), f".oce/exampledataset.oce"))
-    else:
-        dataset =  BaseDataset(data = ExampleDataFrame().to_csv(), structure_col = "Smiles",
-            property_col = "pChEMBL Value") + RandomSplit()
-        save(dataset, path.join(path.expanduser("~"), f".oce/exampledataset.oce"))
-        return dataset
+    dataset = (
+        BaseDataset(
+            data=ExampleDataFrame().to_csv(),
+            structure_col="Smiles",
+            property_col="pChEMBL Value",
+        )
+        + RandomSplit()
+    )
+    return dataset
+
 
 def BACEDataset():
     df = pd.read_csv(download_public_file("MoleculeNet/load_bace_regression.csv"))
-    df["split"] = df["split"].replace({"Train": "train", "Valid": "valid", "Test": "test"})
-    return oce.BaseDataset(data = df.to_csv(), structure_col = "smiles", property_col = "pIC50")
+    df["split"] = df["split"].replace(
+        {"Train": "train", "Valid": "valid", "Test": "test"}
+    )
+    return oce.BaseDataset(
+        data=df.to_csv(), structure_col="smiles", property_col="pIC50"
+    )
+
 
 def test_oce():
     """Convenience function to test all functions of the oce package."""
@@ -134,8 +211,12 @@ def test_oce():
 
     model = BaseBoosting(
         [
-            RandomForestModel(DescriptastorusDescriptor("rdkit2dnormalized"), n_estimators=1),
-            BaseTorchGeometricModel(TLFromCheckpoint("default"), batch_size=8, epochs=1, preinitialized=True),
+            RandomForestModel(
+                DescriptastorusDescriptor("rdkit2dnormalized"), n_estimators=1
+            ),
+            BaseTorchGeometricModel(
+                TLFromCheckpoint("default"), batch_size=8, epochs=1, preinitialized=True
+            ),
             RandomForestModel(OlorenCheckpoint("default"), n_estimators=1),
         ]
     )
@@ -145,4 +226,3 @@ def test_oce():
     save(model, "model.oce")
     _ = load("model.oce")
     os.remove("model.oce")
-
