@@ -1,8 +1,9 @@
-from .GINNetwork.main import GINModel
-from .SPGNN.main import SPGNN
-from .mol2vec.main import Mol2Vec
-from .piCalculax import calc_pI
-from .stoned import *
 from .ChemProp.main import ChemPropModel
+from .GINNetwork.main import GINModel
 from .HondaSTRep.main import HondaSTRep
-from .HondaSTRep.operations import WordVocab
\ No newline at end of file
+from .HondaSTRep.operations import WordVocab
+from .mol2vec.main import Mol2Vec
+from .MolCLR.main import MolCLR, MolCLRVecRep
+from .piCalculax import calc_pI
+from .SPGNN.main import SPGNN, SPGNNVecRep
+from .stoned import *
